<?php

namespace Drupal\crm\Entity;

use Drupal\Core\Entity\EditorialContentEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\crm\CrmContactInterface;
use Drupal\inline_entity_form\Plugin\Field\FieldWidget\InlineEntityFormComplex;
use Drupal\user\EntityOwnerTrait;

/**
 * Defines the contact entity class.
 *
 * @ContentEntityType(
 *   id = "crm_contact",
 *   label = @Translation("CRM Contact"),
 *   label_collection = @Translation("CRM Contacts"),
 *   label_singular = @Translation("crm contact"),
 *   label_plural = @Translation("contacts"),
 *   label_count = @PluralTranslation(
 *     singular = "@count crm contacts",
 *     plural = "@count crm contacts",
 *   ),
 *   bundle_label = @Translation("Contact type"),
 *   handlers = {
 *     "list_builder" = "Drupal\crm\ContactListBuilder",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "access" = "Drupal\crm\ContactAccessControlHandler",
 *     "form" = {
 *       "add" = "Drupal\crm\Form\ContactForm",
 *       "edit" = "Drupal\crm\Form\ContactForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     }
 *   },
 *   base_table = "crm_contact",
 *   revision_table = "crm_contact_revision",
 *   show_revision_ui = TRUE,
 *   admin_permission = "administer crm contact types",
 *   entity_keys = {
 *     "id" = "id",
 *     "revision" = "revision_id",
 *     "bundle" = "bundle",
 *     "label" = "name",
 *     "uuid" = "uuid",
 *     "status" = "status",
 *     "published" = "status",
 *     "owner" = "uid",
 *     "uid" = "uid",
 *   },
 *   revision_metadata_keys = {
 *    "revision_user" = "revision_uid",
 *     "revision_created" = "revision_timestamp",
 *     "revision_log_message" = "revision_log",
 *   },
 *   links = {
 *     "collection" = "/admin/content/crm/contact",
 *     "add-form" = "/crm/contact/add/{crm_contact_type}",
 *     "add-page" = "/crm/contact/add",
 *     "canonical" = "/crm/contact/{crm_contact}",
 *     "edit-form" = "/crm/contact/{crm_contact}/edit",
 *     "delete-form" = "/crm/contact/{crm_contact}/delete",
 *   },
 *   bundle_entity_type = "crm_contact_type",
 *   field_ui_base_route = "entity.crm_contact_type.edit_form",
 * )
 */
class Contact extends EditorialContentEntityBase implements CrmContactInterface {

  use EntityOwnerTrait;

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);
    $this->setNewRevision();

    // If type is person, set the label field to the name field.
    if ($this->bundle() == 'person') {
      $formatted_name = NULL;
      $name_array = $this->get('full_name')->getValue();
      if ($name_array != NULL) {
        $name_formatter = \Drupal::service('name.formatter');
        $formatted_name = $name_formatter->formatList($name_array);
        $this->set('name', $formatted_name);
      }
    }

  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);
    $fields += static::ownerBaseFieldDefinitions($entity_type);

    $fields['primary'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel('Primary Contact')
      ->setSetting('target_type', 'crm_contact')
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 1,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'before',
        'type' => 'entity_reference',
        'weight' => 0,
      ]);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setRevisionable(TRUE)
      ->setLabel(t('Name'))
      ->setRequired(TRUE)
      ->setSetting('max_length', 255)
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['emails'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Emails'))
      ->setRevisionable(TRUE)
      ->setCardinality(-1)
      ->setSetting('target_type', 'crm_contact_detail')
      ->setSetting('handler_settings', ['target_bundles' => ['email']])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'inline_entity_form_complex',
        'weight' => 8,
        'settings' => [
          'allow_new' => TRUE,
          'allow_existing' => FALSE,
          'removed_reference' => InlineEntityFormComplex::REMOVED_DELETE,
          'match_operator' => 'CONTAINS',
          'allow_duplicate' => FALSE,
          'form_mode' => 'default',
          'override_labels' => TRUE,
          'label_singular' => t('email'),
          'label_plural' => t('emails'),
          'collapsible' => FALSE,
          'collapsed' => FALSE,
          'revision' => TRUE,
        ],
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'entity_reference_entity_view',
        'weight' => -5,
        'settings' => [
          'view_mode' => 'default',
          'link' => FALSE,
        ],
      ]);

    $fields['telephones'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Telephones'))
      ->setRevisionable(TRUE)
      ->setCardinality(-1)
      ->setSetting('target_type', 'crm_contact_detail')
      ->setSetting('handler_settings', ['target_bundles' => ['telephone']])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'inline_entity_form_complex',
        'weight' => 9,
        'settings' => [
          'allow_new' => TRUE,
          'allow_existing' => FALSE,
          'removed_reference' => InlineEntityFormComplex::REMOVED_DELETE,
          'match_operator' => 'CONTAINS',
          'allow_duplicate' => FALSE,
          'form_mode' => 'default',
          'override_labels' => TRUE,
          'label_singular' => t('telephone'),
          'label_plural' => t('telephones'),
          'collapsible' => FALSE,
          'collapsed' => FALSE,
          'revision' => TRUE,
        ],
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'entity_reference_entity_view',
        'weight' => -5,
        'settings' => [
          'view_mode' => 'default',
          'link' => FALSE,
        ],
      ]);

    $fields['addresses'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel('Addresses')
      ->setRevisionable(TRUE)
      ->setCardinality(-1)
      ->setSetting('target_type', 'crm_contact_detail')
      ->setSetting('handler_settings', ['target_bundles' => ['address']])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'inline_entity_form_complex',
        'weight' => 10,
        'settings' => [
          'allow_new' => TRUE,
          'allow_existing' => FALSE,
          'removed_reference' => InlineEntityFormComplex::REMOVED_DELETE,
          'match_operator' => 'CONTAINS',
          'allow_duplicate' => FALSE,
          'form_mode' => 'default',
          'override_labels' => TRUE,
          'label_singular' => t('address'),
          'label_plural' => t('addresses'),
          'collapsible' => FALSE,
          'collapsed' => FALSE,
          'revision' => TRUE,
        ],
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'entity_reference_entity_view',
        'weight' => 0,
        'settings' => [
          'view_mode' => 'default',
          'link' => FALSE,
        ],
      ]);

    $fields['uid']
      ->setLabel(t('Owned by'))
      ->setDescription(t('The username of the content author.'))
      ->setRevisionable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['status']
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'settings' => [
          'display_label' => TRUE,
        ],
        'weight' => 120,
      ])
      ->setLabel(t('Status'))
      ->setDefaultValue(TRUE)
      ->setSetting('on_label', 'Status')
      ->setDisplayConfigurable('form', TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created on'))
      ->setDescription(t('The time that the contact was created.'))
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'timestamp',
        'weight' => 20,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'datetime_timestamp',
        'weight' => 20,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the contact was last edited.'));

    return $fields;
  }

}
