<?php

declare(strict_types=1);

namespace Drupal\crm\Entity;

use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\RevisionableContentEntityBase;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\crm\CrmContactDetailInterface;
use Drupal\user\EntityOwnerTrait;

/**
 * Defines the CRM contact detail entity class.
 *
 * @ContentEntityType(
 *   id = "crm_contact_detail",
 *   label = @Translation("CRM Contact Detail"),
 *   label_collection = @Translation("CRM Contact Details"),
 *   label_singular = @Translation("contact detail"),
 *   label_plural = @Translation("contact details"),
 *   label_count = @PluralTranslation(
 *     singular = "@count contact details",
 *     plural = "@count contact details",
 *   ),
 *   bundle_label = @Translation("CRM Contact Detail type"),
 *   handlers = {
 *     "list_builder" = "Drupal\crm\ContactDetailListBuilder",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "form" = {
 *       "add" = "Drupal\Drupal\Core\Entity\ContentEntityForm",
 *       "edit" = "Drupal\Drupal\Core\Entity\ContentEntityForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *     },
 *   },
 *   base_table = "crm_contact_detail",
 *   revision_table = "crm_contact_detail_revision",
 *   show_revision_ui = TRUE,
 *   admin_permission = "administer crm contact detail types",
 *   entity_keys = {
 *     "id" = "id",
 *     "revision" = "revision_id",
 *     "bundle" = "bundle",
 *     "label" = "id",
 *     "uuid" = "uuid",
 *     "status" = "status",
 *     "published" = "status",
 *     "owner" = "uid",
 *     "uid" = "uid",
 *   },
 *   revision_metadata_keys = {
 *     "revision_user" = "revision_uid",
 *     "revision_created" = "revision_timestamp",
 *     "revision_log_message" = "revision_log",
 *   },
 *   bundle_entity_type = "crm_contact_detail_type",
 *   field_ui_base_route = "entity.crm_contact_detail_type.edit_form",
 * )
 */
class ContactDetail extends RevisionableContentEntityBase implements CrmContactDetailInterface {

  use EntityChangedTrait;
  use EntityOwnerTrait;

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage): void {
    parent::preSave($storage);
    if (!$this->getOwnerId()) {
      // If no owner has been set explicitly, make the anonymous user the owner.
      $this->setOwnerId(0);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array {

    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['uid'] = BaseFieldDefinition::create('entity_reference')
      ->setRevisionable(TRUE)
      ->setLabel(t('Author'))
      ->setSetting('target_type', 'user')
      ->setDefaultValueCallback(self::class . '::getDefaultEntityOwner')
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => 60,
          'placeholder' => '',
        ],
        'weight' => 15,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'author',
        'weight' => 15,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['type'] = BaseFieldDefinition::create('entity_reference')
      ->setRevisionable(TRUE)
      ->setLabel(t('Type'))
      ->setSetting('target_type', 'crm_detail_type')
      ->setDisplayOptions('form', [
        'type' => 'options_select',
        'weight' => -10,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'entity_reference_label',
        'weight' => 10,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['status'] = BaseFieldDefinition::create('boolean')
      ->setRevisionable(TRUE)
      ->setLabel(t('Status'))
      ->setDefaultValue(TRUE)
      ->setSetting('on_label', 'Enabled')
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'settings' => [
          'display_label' => FALSE,
        ],
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'type' => 'boolean',
        'label' => 'above',
        'weight' => 0,
        'settings' => [
          'format' => 'enabled-disabled',
        ],
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Authored on'))
      ->setDescription(t('The time that the crm contact detail was created.'))
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'timestamp',
        'weight' => 20,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'datetime_timestamp',
        'weight' => 20,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the crm contact detail was last edited.'));

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    $type = '';

    if ($this->get('type')->isEmpty()) {
      return $type;
    }

    /** @var \Drupal\Core\Entity\Plugin\DataType\EntityReference $detail_type_entity */
    $detail_type_entity_reference = $this->get('type')
      ->first()
      ->get('entity');

    /** @var \Drupal\crm_field\CrmDetailTypeInterface $detail_type_entity */
    $detail_type_entity = $detail_type_entity_reference->getTarget()
      ?->getValue();
    $type = $detail_type_entity?->label();

    return $type;
  }

}
