<?php

namespace Drupal\crm\Entity;

use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityPublishedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\RevisionableContentEntityBase;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\crm\CrmRelationshipInterface;

/**
 * Defines the crm relationship entity class.
 *
 * @ContentEntityType(
 *   id = "crm_relationship",
 *   label = @Translation("CRM Relationship"),
 *   label_collection = @Translation("CRM Relationships"),
 *   label_singular = @Translation("crm relationship"),
 *   label_plural = @Translation("crm relationships"),
 *   label_count = @PluralTranslation(
 *     singular = "@count crm relationships",
 *     plural = "@count crm relationships",
 *   ),
 *   bundle_label = @Translation("CRM Relationship type"),
 *   handlers = {
 *     "list_builder" = "Drupal\crm\RelationshipListBuilder",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "access" = "Drupal\crm\RelationshipAccessControlHandler",
 *     "form" = {
 *       "add" = "Drupal\crm\Form\RelationshipForm",
 *       "edit" = "Drupal\crm\Form\RelationshipForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     }
 *   },
 *   base_table = "crm_relationship",
 *   revision_table = "crm_relationship_revision",
 *   show_revision_ui = TRUE,
 *   translatable = FALSE,
 *   admin_permission = "administer crm relationship types",
 *   entity_keys = {
 *     "id" = "id",
 *     "revision" = "revision_id",
 *     "bundle" = "bundle",
 *     "uuid" = "uuid",
 *     "status" = "status",
 *     "published" = "status",
 *   },
 *   revision_metadata_keys = {
 *    "revision_user" = "revision_uid",
 *     "revision_created" = "revision_timestamp",
 *     "revision_log_message" = "revision_log",
 *   },
 *   links = {
 *     "collection" = "/admin/content/crm/relationship",
 *     "add-form" = "/crm/relationship/add/{crm_relationship_type}",
 *     "add-page" = "/crm/relationship/add",
 *     "canonical" = "/crm/relationship/{crm_relationship}",
 *     "edit-form" = "/crm/relationship/{crm_relationship}/edit",
 *     "delete-form" = "/crm/relationship/{crm_relationship}/delete",
 *   },
 *   bundle_entity_type = "crm_relationship_type",
 *   field_ui_base_route = "entity.crm_relationship_type.edit_form",
 *   constraints = {
 *     "DifferentContacts" = {}
 *   },
 * )
 */
class Relationship extends RevisionableContentEntityBase implements CrmRelationshipInterface {

  use EntityChangedTrait;
  use EntityPublishedTrait;

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {

    $fields = parent::baseFieldDefinitions($entity_type);
    // Add the published field.
    $fields += static::publishedBaseFieldDefinitions($entity_type);

    $fields['contact_a'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel('Contact A')
      ->setSetting('target_type', 'crm_contact')
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'type' => 'entity_reference_label',
        'label' => 'above',
        'weight' => 0,
      ])
      ->setRequired(TRUE);

    $fields['contact_b'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel('Contact B')
      ->setSetting('target_type', 'crm_contact')
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'type' => 'entity_reference_label',
        'label' => 'above',
        'weight' => 0,
      ])
      ->setRequired(TRUE);

    $fields['status'] = BaseFieldDefinition::create('boolean')
      ->setRevisionable(TRUE)
      ->setLabel(t('Status'))
      ->setDefaultValue(TRUE)
      ->setSetting('on_label', 'Status')
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'settings' => [
          'display_label' => FALSE,
        ],
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'type' => 'boolean',
        'label' => 'above',
        'weight' => 0,
        'settings' => [
          'format' => 'enabled-disabled',
        ],
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['start'] = BaseFieldDefinition::create('datetime')
      ->setLabel(t('Start Date'))
      ->setDescription(t('When the relationship started.'))
      ->setRevisionable(TRUE)
      ->setSettings([
        'datetime_type' => 'date',
      ])
      ->setDefaultValue('')
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'datetime_default',
        'settings' => [
          'format_type' => 'medium',
        ],
        'weight' => -9,
      ])
      ->setDisplayOptions('form', [
        'type' => 'datetime_default',
        'weight' => -9,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['end'] = BaseFieldDefinition::create('datetime')
      ->setLabel(t('End Date'))
      ->setDescription(t('When the relationship ends.'))
      ->setRevisionable(TRUE)
      ->setSettings([
        'datetime_type' => 'date',
      ])
      ->setDefaultValue('')
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'datetime_default',
        'settings' => [
          'format_type' => 'medium',
        ],
        'weight' => -9,
      ])
      ->setDisplayOptions('form', [
        'type' => 'datetime_default',
        'weight' => -9,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);
    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created on'))
      ->setDescription(t('The time that the crm relationship was created.'))
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'timestamp',
        'weight' => 20,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'datetime_timestamp',
        'weight' => 20,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the crm relationship was last edited.'));

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    // Get the entity bundle entity.
    $bundle = $this->bundle->entity;
    $label = $bundle->get('label');
    $contact_a = $this->get('contact_a')->entity->label();
    $contact_b = $this->get('contact_b')->entity->label();

    return "$label ($contact_a<=>$contact_b)";
  }

}
