<?php

declare(strict_types=1);

namespace Drupal\Tests\crm\Unit\Entity;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\crm\Entity\Contact;
use Drupal\crm\Entity\Relationship;
use Drupal\crm\Entity\RelationshipType;
use Drupal\Tests\UnitTestCase;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\ContentEntityType;

/**
 * Tests the Relationship entity.
 *
 * @group crm
 * @coversDefaultClass \Drupal\crm\Entity\Relationship
 */
class RelationshipFest extends UnitTestCase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The plugin manager for field types.
   *
   * @var \Drupal\Core\Field\FieldTypePluginManagerInterface
   */
  protected $pluginManagerFieldType;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $container = new ContainerBuilder();

    $this->entityTypeManager = $this->createMock(EntityTypeManagerInterface::class);
    $container->set('entity_type.manager', $this->entityTypeManager);

    $this->entityFieldManager = $this->createMock(EntityFieldManagerInterface::class);
    $container->set('entity_field.manager', $this->entityFieldManager);

    $this->pluginManagerFieldType = $this->createMock('Drupal\Core\Field\FieldTypePluginManagerInterface');
    $container->set('plugin.manager.field.field_type', $this->pluginManagerFieldType);

    \Drupal::setContainer($container);
  }

  /**
   * Tests the Relationship entity.
   *
   * @covers ::baseFieldDefinitions
   */
  public function testBaseFieldDefinitions() {

    $entity_type = new ContentEntityType([
      'id' => 'crm_relationship',
      'label' => 'CRM Relationship',
      'handlers' => [],
      'base_table' => 'crm_relationship',
      'entity_keys' => [
        "id" => "id",
        "revision" => "revision_id",
        "langcode" => "langcode",
        "bundle" => "bundle",
        "uuid" => "uuid",
        "status" => "status",
        "published" => "status",
      ],
      'bundle_entity_type' => 'crm_relationship_type',
    ]);

    $fields = Relationship::baseFieldDefinitions($entity_type);

    $this->assertCount(5, $fields);
    $this->assertArrayHasKey('contact_a', $fields);
    $this->assertArrayHasKey('contact_b', $fields);
    $this->assertArrayHasKey('status', $fields);
    $this->assertArrayHasKey('start', $fields);
    $this->assertArrayHasKey('end', $fields);
  }

  /**
   * Tests the label method.
   *
   * @covers ::label
   */
  public function testLabel() {

    $entity_type_definition = $this->createMock(EntityTypeInterface::class);
    $entity_type_definition->expects($this->exactly(4))
      ->method('getKey')
      ->willReturnMap([
      ['bundle', 'bundle'],
      ['uuid', 'uuid'],
      ]);

    // Mock the entity type manager to return the definition.
    $this->entityTypeManager
      ->method('getDefinition')
      ->with('crm_relationship')
      ->willReturn($entity_type_definition);

    $bundle = $this->createMock(RelationshipType::class);
    $bundle->method('label')
      ->willReturn('Employment');

    $contact_a = $this->createMock(Contact::class);
    $contact_a->method('label')
      ->willReturn('John Doe');
    $contact_b = $this->createMock(Contact::class);
    $contact_b->method('label')
      ->willReturn('Department of Labor');

    $relationship = new Relationship([
      'contact_a' => $contact_a,
      'contact_b' => $contact_b,
      'status' => TRUE,
      'start' => strtotime('2023-01-01'),
      'end' => strtotime('2023-12-31'),
    ], 'crm_relationship', 'employment');

    $relationship->set('bundle', $bundle);

    $this->assertEquals('Test Relationship', $relationship->label());
  }

}
